var numberIsNan = Number.isNaN || function (x) {
	return x !== x;
};

function _objectEntries(obj) {
  var entries = [];
  var keys = Object.keys(obj);

  for (var k = 0; k < keys.length; k++) entries.push([keys[k], obj[keys[k]]]);

  return entries;
}
var SEPARATOR_CHARACTER = '&';
var EQUALS_CHARACTER = '=';
var SEPARATOR_CHARACTER_REGEX = new RegExp(SEPARATOR_CHARACTER, 'g');
var ESCAPED_STRING_REGEX = /^_escape=/;
/**
 * @param  {string}   searchParamsString
 * @param  {string}   splitWith
 * @param  {string[]}   replaceWith
 * @param  {string}   joinWith
 * @param  {Function} callback
 *
 * @returns {string}
 */

function customQueryStringParser(searchParamsString, splitWith, replaceWith, joinWith, callback) {
  if (callback === void 0) {
    callback = function callback(substring) {
      return substring;
    };
  }

  return searchParamsString.split(splitWith).map(function (substring) {
    return callback(substring.replace(replaceWith[0], replaceWith[1]));
  }).join(joinWith);
}
/**
 * @param  {*} input
 * @param  {string} separator
 * @param  {string} equals
 *
 * @returns {string}
 */


function stringify(input, separator, equals) {
  if (separator === void 0) {
    separator = SEPARATOR_CHARACTER;
  }

  if (equals === void 0) {
    equals = EQUALS_CHARACTER;
  }

  if (typeof input === 'undefined' || typeof input === 'number' || Array.isArray(input) || input === null || [false, true].indexOf(input) !== -1) {
    return '';
  }

  var searchParams = new URLSearchParams();

  _objectEntries(input).map(function (_ref) {
    var key = _ref[0],
        value = _ref[1];

    if (value === Infinity || numberIsNan(value) || typeof value === 'function' || typeof value === 'object' && !Array.isArray(value)) {
      return [key, ''];
    }

    return [key, value];
  }).forEach(function (_ref2) {
    var key = _ref2[0],
        value = _ref2[1];

    if (Array.isArray(value)) {
      value.forEach(function (arrayItem) {
        searchParams.append(key, arrayItem);
      });
    } else {
      searchParams.set(key, value);
    }
  });

  var searchParamsString = searchParams.toString();

  if (separator !== SEPARATOR_CHARACTER || equals !== EQUALS_CHARACTER) {
    searchParamsString = customQueryStringParser(searchParamsString, SEPARATOR_CHARACTER, [EQUALS_CHARACTER, equals], separator);
  }

  return searchParamsString;
}
/**
 * @param  {string} input
 * @param  {string} separator
 * @param  {string} equals
 *
 * @returns {object}
 */


function parse(input, separator, equals) {
  if (separator === void 0) {
    separator = SEPARATOR_CHARACTER;
  }

  if (equals === void 0) {
    equals = EQUALS_CHARACTER;
  }

  var searchParamsObject = Object.create(null);

  if (input === null || typeof input === 'undefined') {
    return searchParamsObject;
  }

  var searchParamsString = input;

  if (separator !== SEPARATOR_CHARACTER || equals !== EQUALS_CHARACTER) {
    var encodedSeparatorCharacter = encodeURIComponent(SEPARATOR_CHARACTER);
    searchParamsString = customQueryStringParser(searchParamsString, separator, [equals, EQUALS_CHARACTER], SEPARATOR_CHARACTER, function (substring) {
      SEPARATOR_CHARACTER_REGEX.lastIndex = 0;
      return substring.replace(SEPARATOR_CHARACTER_REGEX, encodedSeparatorCharacter);
    });
  }

  var searchParams = new URLSearchParams(searchParamsString);
  searchParams.forEach(function (value, key) {
    searchParamsObject[key] = null;
  });
  Object.keys(searchParamsObject).forEach(function (key) {
    searchParamsObject[key] = searchParams.getAll(key);
  });

  var filteredSearchParamsObject = _objectEntries(searchParamsObject).map(function (_ref3) {
    var key = _ref3[0],
        value = _ref3[1];

    if (Array.isArray(value) && value.length === 1) {
      return [key, value[0]];
    }

    return [key, value];
  }).reduce(function (object_, _ref4) {
    var key = _ref4[0],
        value = _ref4[1];
    object_[key] = value;
    return object_;
  }, Object.create(null));

  return filteredSearchParamsObject;
}
/**
 * @param  {*} input
 *
 * @returns {string}
 */


function qsEscape(input) {
  var searchParams = new URLSearchParams({
    _escape: input
  });
  return searchParams.toString().replace(ESCAPED_STRING_REGEX, '');
}
/**
 * @param  {*} input
 *
 * @returns {string}
 */


function qsUnescape(input) {
  var searchParams = new URLSearchParams("_unescape=" + input);
  return searchParams.get('_unescape');
}

export { parse as decode, stringify as encode, qsEscape as escape, parse, stringify, qsUnescape as unescape };
//# sourceMappingURL=index.js.map
